<?php

namespace Drupal\Tests\popular_tags\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\taxonomy\Traits\TaxonomyTestTrait;
use Drupal\Tests\field\Traits\EntityReferenceTestTrait;

class PopularTagsWidgetTest extends BrowserTestBase {

  use TaxonomyTestTrait;
  use EntityReferenceTestTrait;

 /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  protected $fieldName;
  protected $contentType;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = ['taxonomy', 'node', 'user', 'field_ui', 'popular_tags'];

  protected function setUp(): void {
    parent::setUp();

    $vocabulary = $this->createVocabulary();

    $contentType = $this->createContentType([
      'type' => 'ptag_page',
      'name' => 'Popular Tag Page'
    ]);
    $this->contentType = $contentType->id();

    $field_name = 'field_' . $vocabulary->id();
    $this->fieldName = $field_name;

    $handler_settings = [
      'target_bundles' => [
        $vocabulary->id() => $vocabulary->id(),
      ],
      'auto_create' => TRUE,
    ];
    $this->createEntityReferenceField('node', $contentType->id(), $field_name, 'Tags', 'taxonomy_term', 'default', $handler_settings, 5);

    $display_repository = \Drupal::service('entity_display.repository');
    $display_repository->getFormDisplay('node', $contentType->id())
      ->setComponent($field_name, [
        'type' => 'entity_reference_autocomplete_tags',
        'weight' => -4,
      ])
      ->save();

    $account = $this->createUser(['administer node form display']);
    $this->drupalLogin($account);

  }

  public function testAutocompleteTagsWidget() {
    $this->drupalGet("/admin/structure/types/manage/{$this->contentType}/form-display");

    $session_assert = $this->assertSession();
    // Confirm the settings form exists.
    $session_assert->statusCodeEquals(200);

    $selector = 'input[name="' . $this->fieldName . '_settings_edit"]';
    $session_assert->elementExists('css', $selector);
    $this->click($selector);
    $session_assert->statusCodeEquals(200);
    $session_assert->pageTextContains('Popular Tags');

    $base_selector = 'input[name="fields[' . $this->fieldName . '][settings_edit_form][third_party_settings][popular_tags][popular_tags][REPLACE_THIS]"]';
    $selectors = [
      str_replace('REPLACE_THIS', 'use', $base_selector),
      str_replace('REPLACE_THIS', 'limit', $base_selector),
      str_replace('REPLACE_THIS', 'showhide', $base_selector),
    ];
    // Make sure each of the expected elements is present.
    foreach($selectors as $selector) {
      $session_assert->elementExists('css', $selector);
    }
  }

}